from .exceptions import ConfigurationError


class Environment(object):
    def __init__(self, name, base_url, qr_url):
        self.name = name
        self.base_url = base_url
        self.qr_url = qr_url

    def __str__(self):
        return self.name

    @staticmethod
    def parse_environment(environment):
        if isinstance(environment, Environment) or environment is None:
            return environment
        try:
            return Environment.All[environment]
        except KeyError:
            raise ConfigurationError("Provided environment name is invalid")


Environment.MSS = Environment(
    name="mss",
    base_url="https://mss.cpc.getswish.net/swish-cpcapi/api/v1/",
    qr_url=None ## MSS doesn't need or have QR codes
)
Environment.Sandbox = Environment(
    name="sandbox",
    base_url="https://staging.getswish.pub.tds.tieto.com/swish-cpcapi/api/v1/",
    qr_url="https://staging.getswish.pub.tds.tieto.com/qrg-swish/api/v1/"
)
Environment.Production = Environment(
    name="production",
    base_url="https://cpc.getswish.net/swish-cpcapi/api/v1/",
    qr_url="https://mpc.getswish.net/qrg-swish/api/v1/"
)

# deprecated
Environment.Test = Environment(
    name="test",
    base_url="https://mss.cpc.getswish.net/swish-cpcapi/api/v1/",
    qr_url=None
)

Environment.All = {
    "test": Environment.Test,
    "mss": Environment.MSS,
    "sandbox": Environment.Sandbox,
    "production": Environment.Production
}



import warnings

import requests

from .environment import Environment
from .exceptions import SwishError
from .models import Payment, Refund, CommerceQRCodeRequest

try:
    from requests.packages.urllib3.contrib import pyopenssl
    pyopenssl.extract_from_urllib3()
except ImportError:
    pass


class SwishClient(object):
    def __init__(self, environment, merchant_swish_number, cert, verify=False):
        self.environment = Environment.parse_environment(environment)
        self.merchant_swish_number = merchant_swish_number
        self.cert = cert
        self.verify = verify

    def __post(self, endpoint, payload):
        if endpoint == 'commerce':
          base_url = self.environment.qr_url
        else:
          base_url = self.environment.base_url
        url = base_url + endpoint
        return requests.post(url=url, json=payload.to_primitive(), headers={'Content-Type': 'application/json'},
                             cert=self.cert, verify=self.verify)

    def post(self, endpoint, payload):
        warnings.warn("You shouldn't be calling this method. It will be private in a future version.", DeprecationWarning);
        self.__post(endpoint, payload)

    def __get(self, endpoint, parameter):
        url = self.environment.base_url + endpoint + '/' + str(parameter)
        return requests.get(url, cert=self.cert, verify=self.verify)

    def get(self, endpoint, parameter):
        warnings.warn("You shouldn't be calling this method. It will be private in a future version.", DeprecationWarning);
        self.__get(endpoint, parameter)

    def create_payment(self, amount, currency, callback_url, payee_payment_reference=None, message=None,
                       payer_alias=None):
        payment_request = Payment({
            'payee_alias': self.merchant_swish_number,
            'amount': amount,
            'currency': currency,
            'callback_url': callback_url,
            'payee_payment_reference': payee_payment_reference,
            'message': message,
            'payer_alias': payer_alias
        })

        response = self.__post('paymentrequests', payment_request)
        if response.status_code == 422:
            raise SwishError(response.json())
        response.raise_for_status()

        return Payment({'id': response.headers.get('Location').split('/')[-1],
                        'location': response.headers.get('Location'),
                        'request_token': response.headers.get('PaymentRequestToken')})

    def get_payment(self, payment_request_id):
        response = self.__get('paymentrequests', payment_request_id)
        response.raise_for_status()
        return Payment(response.json())

    def create_refund(self, original_payment_reference, amount, currency, callback_url, payer_payment_reference=None,
                      payment_reference=None, payee_alias=None, message=None):
        refund_request = Refund({
            'payer_alias': self.merchant_swish_number,
            'payee_alias': payee_alias,
            'original_payment_reference': original_payment_reference,
            'amount': amount,
            'currency': currency,
            'callback_url': callback_url,
            'payer_payment_reference': payer_payment_reference,
            'payment_reference': payment_reference,
            'message': message
        })

        response = self.__post('refunds', refund_request)
        if response.status_code == 422:
            raise SwishError(response.json())
        response.raise_for_status()

        return Refund({'id': response.headers.get('Location').split('/')[-1],
                       'location': response.headers.get('Location')})

    def get_refund(self, refund_id):
        response = self.__get('refunds', refund_id)
        response.raise_for_status()
        return Refund(response.json())


    def commerce_qr_code(self, token, format, size=None, border=None, transparent=None):
        commerce_qr_code_request = CommerceQRCodeRequest({
            'token': token,
            'format': format,
            'size': size,
            'border': border,
            'transparent': transparent
        })

        response = self.__post('commerce', commerce_qr_code_request)
        response.raise_for_status()
        return response.content



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'LONG_DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='swish',
    version='1.4',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description='Swish Python Client Library',
    long_description=long_description,
    url='https://github.com/MadSpindel/swish-python',
    author='Fredrik Sundqvist',
    author_email='fsundqvist@gmail.com',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        "Topic :: Software Development :: Libraries :: Python Modules",
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    install_requires=['requests>=2.9.1', 'schematics>=2.0.0.dev2'],
    zip_safe=False
)
